@@ -13,6 +13,11 @@ def frame_count_to_time(frame_count_input, frame_rate_input):
     return (frame_count_input / frame_rate_input) * 1000
 
 
+# This function is to change time in millisecond to frame count
+def time_to_frame_count(time_to_calculate, frame_rate_input):
+    return int((time_to_calculate / 1000) * frame_rate_input)
+
+
 # This function is to read string to convert boolean value
 def str_to_bool(input_bool_string):
     if str(input_bool_string).lower().strip() == "true":
@@ -141,35 +146,60 @@ def get_color_position(image_input, color_in):
 
 
 # This function is to create the data array in order to draw lines in graph
-def get_data_array(csv_data_input, t_data_input, y_data_input,
+def get_data_array(csv_data_input, t_data_input, y_data_input, offset_input,
                    filter_input=False, filter_by_input=None):
     if filter_input:
-        filter_name, filter_value = str(filter_by_input).split("=")
-        filter_value = int(filter_value)
-        data_array_output = []
-        row_count = 0
-        with open(csv_data_input, "r") as csv_file:
-            csv_reader = (csv.reader(csv_file, delimiter=','))
-            header_array = next(csv_reader)
-            t_pos = get_position(t_data_input, header_array)
-            y_pos = get_position(y_data_input, header_array)
-            f_pos = get_position(filter_name, header_array)
+        if "=" in filter_by_input:
+            filter_name, filter_value = str(filter_by_input).split("=")
+            filter_value = int(filter_value)
+            data_array_output = []
+            with open(csv_data_input, "r") as csv_file:
+                csv_reader = (csv.reader(csv_file, delimiter=','))
+                header_array = next(csv_reader)
+                t_pos = get_position(t_data_input, header_array)
+                y_pos = get_position(y_data_input, header_array)
+                f_pos = get_position(filter_name, header_array)
 
-            for row in csv_reader:
-                if row_count == 0:
-                    row_count += 1
-                    pass
-                else:
+                for row in csv_reader:
                     filter_check = int(row[f_pos])
                     if filter_check == filter_value:
-                        time_value = float(row[t_pos]) * 1000
+                        time_value = (float(row[t_pos]) + offset_input) * 1000
                         y_value = float(row[y_pos])
                         data_array_output.append([time_value, y_value])
 
-        return data_array_output
+            return data_array_output
+
+        elif "event" in filter_by_input:
+            data_array_output = []
+            experience_data = False
+            first_value_record = False
+
+            with open(csv_data_input, "r") as csv_file:
+                csv_reader = (csv.reader(csv_file, delimiter=','))
+                header_array = next(csv_reader)
+                t_pos = get_position(t_data_input, header_array)
+                y_pos = get_position(y_data_input, header_array)
+                f_pos = get_position(filter_by_input, header_array)
+
+                for row in csv_reader:
+                    event_string_value = str(row[f_pos])
+                    if event_string_value != " ":
+                        experience_data = True
+                        if not first_value_record:
+                            first_time_value = (float(row[t_pos]) * 1000)
+                            first_value_record = True
+                        if event_string_value == "server_end_of_file_message":
+                            experience_data = False
+                    if experience_data and first_value_record:
+                        original_time_value = (float(row[t_pos]) * 1000)
+                        time_value = ((float(row[t_pos]) + offset_input) * 1000) - first_time_value
+                        y_value = float(row[y_pos])
+                        data_array_output.append([time_value, y_value, original_time_value])
+            # print(len(data_array_output))
+            return data_array_output
+
     else:
         data_array_output = []
-        row_count = 0
         with open(csv_data_input, "r") as csv_file:
             csv_reader = (csv.reader(csv_file, delimiter=','))
             header_array = next(csv_reader)
@@ -177,13 +207,9 @@ def get_data_array(csv_data_input, t_data_input, y_data_input,
             y_pos = get_position(y_data_input, header_array)
 
             for row in csv_reader:
-                if row_count == 0:
-                    row_count += 1
-                    pass
-                elif row_count == 1:
-                    time_value = float(row[t_pos]) * 1000
-                    y_value = float(row[y_pos])
-                    data_array_output.append([time_value, y_value])
+                time_value = (float(row[t_pos]) + offset_input) * 1000
+                y_value = float(row[y_pos])
+                data_array_output.append([time_value, y_value])
 
         return data_array_output
 
@@ -212,20 +238,26 @@ def get_value_from_array(data_arr_input, low_li, up_li):
     if length_array == 1:
         val = new_arr[0]
         return val
+
     elif length_array == 0:
         val = [((low_li + up_li) / 2), "nothing"]
         return val
+
     else:
-        avg_time = sum(new_arr[0]) / len(new_arr[0])
-        avg_value = sum(new_arr[1]) / len(new_arr[1])
+        total_time = 0
+        total_value = 0
+        for d in new_arr:
+            total_time += d[0]
+            total_value += d[1]
+        avg_time = total_time / length_array
+        avg_value = total_value / length_array
         val = [avg_time, avg_value]
         return val
 
 
 def main():
-    parser = argparse.ArgumentParser(prog='vinset',
-                                     description='VINSET package.')
-    parser.add_argument('--version', action='version', version='2.0.9'),
+    parser = argparse.ArgumentParser(prog='vinset', description='VINSET package.')
+    parser.add_argument('--version', action='version', version='3.0.2'),
     parser.add_argument("-i", dest="input_filename", required=True, type=argparse.FileType('r'), default=sys.stdin,
                         help="input mp4 file", metavar="filename.mp4")
     parser.add_argument("-d", dest="input_data_filename", required=True, type=argparse.FileType('r'), default=sys.stdin,
@@ -284,6 +316,7 @@ def main():
         print("Frame count:", frame_count)
 
         draw_data_array = []
+        pointer_info_dict = {}
         max_video_length = math.ceil(frame_count_to_time(frame_count, frame_rate))
 
         for info in series_array:
@@ -297,8 +330,14 @@ def main():
                 axes_pos_w = axes_pos["width"]
                 # axes_pos_h = axes_pos["height"]
                 series_name = info["name"]
+                pointer_position_name = series_name + "_pointer_info"
+                pointer_info_dict[pointer_position_name] = (0, 0)
                 display_type = info["display_type"]
                 t_limit = info["t-limit"]
+                try:
+                    t_offset = info["time_offset"]
+                except KeyError:
+                    t_offset = 0
                 t_w = t_limit["width"]
                 t_data = info["t_data"]
                 y_data = info["y_data"]
@@ -308,9 +347,9 @@ def main():
                     filter_by = info["filterBy"]
                 if str(display_type).lower() == "pen":
                     if info_filter:
-                        data_arr = get_data_array(csv_data, t_data, y_data, info_filter, filter_by)
+                        data_arr = get_data_array(csv_data, t_data, y_data, t_offset, info_filter, filter_by)
                     else:
-                        data_arr = get_data_array(csv_data, t_data, y_data)
+                        data_arr = get_data_array(csv_data, t_data, y_data, t_offset)
                     series_data_array = []
                     time_interval_for_data = round(1000 / (axes_pos_w / t_w), 4)
                     number_of_time_interval = math.ceil(max_video_length / time_interval_for_data)
@@ -338,18 +377,19 @@ def main():
                                 width_check += (int(t_w) * 1000)
                         lower_range = upper_range
                         upper_range = round((upper_range + time_interval_for_data), 2)
-                    draw_data_array.append([series_name, display_type, series_data_array])
+                    draw_data_array.append({"series_name": series_name, "display_type": display_type,
+                                            "series_data_array": series_data_array})
 
                 elif str(display_type).lower() == "static":
                     if info_filter:
-                        data_arr = get_data_array(csv_data, t_data, y_data, info_filter, filter_by)
+                        data_arr = get_data_array(csv_data, t_data, y_data, t_offset, info_filter, filter_by)
                     else:
-                        data_arr = get_data_array(csv_data, t_data, y_data)
+                        data_arr = get_data_array(csv_data, t_data, y_data, t_offset)
                     series_data_array = []
                     time_interval_for_data = round(1000 / (axes_pos_w / t_w), 4)
                     number_of_time_interval = math.ceil(max_video_length / time_interval_for_data)
                     lower_range = 0
-                    upper_range = round((lower_range + time_interval_for_data), 2)
+                    upper_range = round((lower_range + time_interval_for_data), 2) * 10
                     width_check = int(t_w) * 1000
                     box_data_array = []
                     for i in range(1, (number_of_time_interval + 1), 1):
@@ -359,6 +399,7 @@ def main():
                             if lower_range <= data[0] < upper_range:
                                 raw_data_array.append(data)
                         input_data = get_value_from_array(raw_data_array, lower_range, upper_range)
+                        raw_data_array.clear()
                         box_data_array.append(input_data)
                         if max_video_length <= width_check:
                             if i == (number_of_time_interval - 1):
@@ -372,7 +413,8 @@ def main():
                                 width_check += (int(t_w) * 1000)
                         lower_range = upper_range
                         upper_range = round((upper_range + time_interval_for_data), 2)
-                    draw_data_array.append([series_name, display_type, series_data_array])
+                    draw_data_array.append({"series_name": series_name, "display_type": display_type,
+                                            "series_data_array": series_data_array})
 
         ret, frame = input_video.read()
         raw_image = None
@@ -380,7 +422,6 @@ def main():
             raw_image = frame
             raw_image.fill(0)
         box_color_array = []
-        line_color_array = []
 
         for info in series_array:
             search_string = info["parent_axes"]
@@ -391,6 +432,9 @@ def main():
                 box_color_array.append(box_color)
                 axes_pos = axes_info["position"]
                 box_title = axes_info["box_title"]
+                title_x_position = axes_info["box_title_x_position"]
+                title_y_position = axes_info["box_title_y_position"]
+                title_color = change_string_to_color_tuple(axes_info["box_title_color"])
                 title_font = float(axes_info["box_title_font_scale"])
                 box_thickness = int(axes_info["box_thickness"])
                 axes_pos_x = axes_pos["x"]
@@ -400,28 +444,26 @@ def main():
                 t_limit = info["t-limit"]
                 t_w = t_limit["width"]
                 t_name = info["t_label"]
+                t_label_x_position = info["t_label_x_position"]
+                t_label_y_position = info["t_label_y_position"]
                 y_name = info["y_label"]
+                y_label_x_position = info["y_label_x_position"]
+                y_label_y_position = info["y_label_y_position"]
                 label_thickness = int(info["label_thickness"])
                 label_font_scale = float(info["label_font_scale"])
-                line_color = info["line_color"]
-                line_color_array.append(change_string_to_color_tuple(line_color))
                 raw_image = cv2.rectangle(raw_image, (axes_pos_x, axes_pos_y),
                                           ((axes_pos_x + axes_pos_w), (axes_pos_y + axes_pos_h)),
                                           box_color, box_thickness)
-                offset_value = 30
-                box_title_length = len(box_title)
-                title_x = int(axes_pos_x) + int(axes_pos_w / 2) - (5 * box_title_length)
-                title_y = int(axes_pos_y) - int(offset_value / 2)
+                title_x = int(axes_pos_x) + int(title_x_position)
+                title_y = int(axes_pos_y) - int(title_y_position)
                 cv2.putText(raw_image, box_title, (title_x, title_y), cv2.FONT_HERSHEY_SIMPLEX, title_font,
-                            box_color, box_thickness)
-                t_name_length = len(t_name)
-                t_label_x = int(axes_pos_x) + int(axes_pos_w / 2) - (5 * t_name_length)
-                t_label_y = int(axes_pos_y) + int(axes_pos_h) + offset_value
+                            title_color, box_thickness)
+                t_label_x = int(axes_pos_x) + int(t_label_x_position)
+                t_label_y = int(axes_pos_y) + int(t_label_y_position)
                 cv2.putText(raw_image, t_name, (t_label_x, t_label_y), cv2.FONT_HERSHEY_SIMPLEX, label_font_scale,
                             box_color, label_thickness)
-                y_name_length = len(y_name)
-                y_label_x_input = axes_pos_x - offset_value + 10
-                y_label_y_input = axes_pos_y - (int(axes_pos_h / 2) + offset_value) + (1 * y_name_length)
+                y_label_x_input = axes_pos_x - int(y_label_x_position)
+                y_label_y_input = axes_pos_y - int(y_label_y_position)
                 ver_color_index_array = get_vertical_position(frame_width, frame_height, y_name,
                                                               y_label_x_input, y_label_y_input,
                                                               box_color, label_font_scale, label_thickness)
@@ -452,8 +494,9 @@ def main():
         v_writer = cv2.VideoWriter(output_file, fourcc, int(frame_rate), (int(frame_width), int(frame_height)))
         whole_t = time.time()
         count = 0
+        drawing_graph = True
 
-        while True:
+        while drawing_graph:
 
             # start_tt = time.time()
             ret, frame = input_video.read()
@@ -492,179 +535,199 @@ def main():
 
                 clone_frame = np.copy(frame)
 
-                for i, info in enumerate(series_array):
-                    display_arr = draw_data_array[i][2]
-                    # info_name = info["name"]
-                    search_str = info["parent_axes"]
-                    pointer_info = info["pointer_value"]
-                    need_to_draw_pointer = bool(pointer_info["Enabled"])
-                    line_thickness = int(info["line_thickness"])
-                    zero_display = str_to_bool(str(info["zero_line_display"]))
-                    zero_line_thick = 0
-                    if zero_display:
-                        zero_line_thick = int(info["zero_line_thickness"])
-                    show_type = str(info["display_type"])
-                    info_y_limit = info["y-limit"]
-                    info_limits = info_y_limit["limits"]
-                    lower_limit = int(info_limits["lower"])
-                    upper_limit = int(info_limits["upper"])
-                    zero_display = need_to_display_zero_line(lower_limit, upper_limit)
-                    info_t_limit = info["t-limit"]
-                    t_width = info_t_limit["width"]
-                    pointer_color = None
-                    pointer_radius = None
-                    if need_to_draw_pointer:
-                        pointer_color = change_string_to_color_tuple(pointer_info["Color"])
-                        pointer_radius = pointer_info["Radius"]
-                    parent_index = get_position_of_parent(search_str, axes_array)
-                    axes_info = axes_array[parent_index]
-                    if show_type == "pen" and parent_index is not None:
-                        ax_pos = axes_info["position"]
-                        ax_x = ax_pos["x"]
-                        ax_y = ax_pos["y"]
-                        ax_w = ax_pos["width"]
-                        ax_h = ax_pos["height"]
-                        zero_line_start_x = ax_x
-                        zero_line_y_scale = int(number_to_scale(lower_limit, upper_limit, ax_h, 0))
-                        zero_line_y = int(ax_y + ax_h - zero_line_y_scale)
-                        zero_line_end_x = ax_x + ax_w
-                        time_in_pixel_value = round((t_width / ax_w), 20)
-                        time_interval_for_data = round(1 / (ax_w / t_width), 20)
-                        start_x = ax_x
-                        start_y = 0
-                        value_to_draw = False
-                        display_count = int(check_timer / (t_width * 1000))
-                        # total_display_count = int(max_video_length / (t_width * 1000))
-                        # end_x = 0
-                        # end_y = 0
-                        final_display_arr = display_arr[display_count][1]
-                        draw_after = False
-                        pointer_x = 0
-                        pointer_y = 0
-                        check_timer_input = check_timer - (display_count * t_width * 1000)
-                        drawable_limit = ax_x + int((check_timer_input / (time_interval_for_data * 1000)))
-                        # pen_draw = False
-                        no_first_value = False
+                for info in series_array:
+                    if len(draw_data_array) > 0:
+                        info_name = info["name"]
+                        display_arr = None
+                        for dic in draw_data_array:
+                            if dic["series_name"] == info_name:
+                                display_arr = dic["series_data_array"]
+                                break
+                        search_str = info["parent_axes"]
+                        pointer_info = info["pointer_value"]
+                        need_to_draw_pointer = bool(pointer_info["Enabled"])
+                        line_color = change_string_to_color_tuple(info["line_color"])
+                        line_thickness = int(info["line_thickness"])
+                        zero_display = str_to_bool(str(info["zero_line_display"]))
+                        zero_line_thick = 0
+                        if zero_display:
+                            zero_line_thick = int(info["zero_line_thickness"])
+                        show_type = str(info["display_type"])
+                        info_y_limit = info["y-limit"]
+                        info_limits = info_y_limit["limits"]
+                        lower_limit = info_limits["lower"]
+                        upper_limit = info_limits["upper"]
+                        can_zero_line_be_drawn = need_to_display_zero_line(lower_limit, upper_limit)
+                        info_t_limit = info["t-limit"]
+                        t_width = info_t_limit["width"]
+                        pointer_color = None
+                        pointer_radius = None
+                        if need_to_draw_pointer:
+                            pointer_color = change_string_to_color_tuple(pointer_info["Color"])
+                            pointer_radius = pointer_info["Radius"]
+                        parent_index = get_position_of_parent(search_str, axes_array)
+                        axes_info = axes_array[parent_index]
+                        if show_type == "pen" and parent_index is not None:
+                            ax_pos = axes_info["position"]
+                            ax_x = ax_pos["x"]
+                            ax_y = ax_pos["y"]
+                            ax_w = ax_pos["width"]
+                            ax_h = ax_pos["height"]
+                            if can_zero_line_be_drawn:
+                                zero_line_start_x = ax_x
+                                zero_line_y_scale = int(number_to_scale(lower_limit, upper_limit, ax_h, 0))
+                                zero_line_y = int(ax_y + ax_h - zero_line_y_scale)
+                                zero_line_end_x = ax_x + ax_w
+                            else:
+                                zero_line_start_x = 0
+                                zero_line_y_scale = 0
+                                zero_line_y = 0
+                                zero_line_end_x = 0
+                            time_in_pixel_value = round((t_width / ax_w), 20)
+                            # print(time_in_pixel_value)
+                            time_interval_for_data = round(1 / (ax_w / t_width), 20)
+                            start_x = ax_x
+                            start_y = 0
+                            value_to_draw = False
+                            display_count = int(check_timer / (t_width * 1000))
+                            total_display_count = int(max_video_length / (t_width * 1000))
+                            end_x = 0
+                            end_y = 0
+                            final_display_arr = display_arr[display_count][1]
+                            pointer_x = 0
+                            pointer_y = 0
+                            check_timer_input = check_timer - (display_count * t_width * 1000)
+                            drawable_limit = ax_x + int((check_timer_input / (time_interval_for_data * 1000)))
+                            pen_draw = False
+                            no_first_value = False
+                            pointer_position_info_name = info_name + "_pointer_info"
 
-                        if len(final_display_arr) > 0:
-                            for ind, num in enumerate(final_display_arr):
-                                if zero_display:
-                                    cv2.line(clone_frame, (zero_line_start_x, zero_line_y),
-                                             (zero_line_end_x, zero_line_y),
-                                             line_color_array[i], zero_line_thick)
-                                if type(num[1]) == str:
-                                    if ind == 0:
-                                        no_first_value = True
+                            if len(final_display_arr) > 0:
+                                for ind, num in enumerate(final_display_arr):
+                                    if zero_display:
+                                        cv2.line(clone_frame, (zero_line_start_x, zero_line_y),
+                                                 (zero_line_end_x, zero_line_y),
+                                                 line_color, zero_line_thick)
+                                    if type(num[1]) == str:
+                                        if ind == 0:
+                                            no_first_value = True
 
-                                elif math.isnan(num[1]):
-                                    value_to_draw = False
+                                    elif math.isnan(num[1]):
+                                        value_to_draw = False
 
-                                else:
-                                    if not value_to_draw:
-                                        if no_first_value:
-                                            scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
-                                            start_x = ax_x
-                                            start_y = int(ax_y + ax_h - scaled_num)
-                                            value_to_draw = True
-                                            no_first_value = False
+                                    else:
+                                        if not value_to_draw:
+                                            if no_first_value:
+                                                scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
+                                                start_x = ax_x
+                                                start_y = int(ax_y + ax_h - scaled_num)
+                                                value_to_draw = True
+                                                no_first_value = False
+                                            else:
+                                                scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
+                                                # scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
+                                                scaled_step = math.ceil((num[0] / 1000) / time_in_pixel_value)
+                                                start_x = ax_x + scaled_step - (display_count * ax_w)
+                                                start_y = int(ax_y + ax_h - scaled_num)
+                                                value_to_draw = True
                                         else:
+                                            if start_x >= drawable_limit:
+                                                pen_draw = False
+                                            else:
+                                                pen_draw = True
+                                            if pen_draw:
+                                                scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
+                                                # scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
+                                                scaled_step = math.ceil((num[0] / 1000) / time_in_pixel_value)
+                                                end_x = ax_x + scaled_step - (display_count * ax_w)
+                                                end_y = int(ax_y + ax_h - scaled_num)
+                                                cv2.line(clone_frame, (start_x, start_y), (end_x, end_y),
+                                                         line_color, line_thickness, cv2.LINE_AA)
+                                                if need_to_draw_pointer and round(num[0] / 100, 0) >= round(
+                                                        check_timer / 100, 0):
+                                                    pointer_info_dict[pointer_position_info_name] = (end_x, end_y)
+
+                                                start_x = end_x
+                                                start_y = end_y
+
+                                pointer_position_tuple = pointer_info_dict[pointer_position_info_name]
+                                pointer_x = pointer_position_tuple[0]
+                                pointer_y = pointer_position_tuple[1]
+                                cv2.circle(clone_frame, (pointer_x, pointer_y), pointer_radius, pointer_color, -1)
+
+                        elif show_type == "static" and parent_index is not None:
+                            ax_pos = axes_info["position"]
+                            ax_x = ax_pos["x"]
+                            ax_y = ax_pos["y"]
+                            ax_w = ax_pos["width"]
+                            ax_h = ax_pos["height"]
+                            if can_zero_line_be_drawn:
+                                zero_line_start_x = ax_x
+                                zero_line_y_scale = int(number_to_scale(lower_limit, upper_limit, ax_h, 0))
+                                zero_line_y = int(ax_y + ax_h - zero_line_y_scale)
+                                zero_line_end_x = ax_x + ax_w
+                            else:
+                                zero_line_start_x = 0
+                                zero_line_y_scale = 0
+                                zero_line_y = 0
+                                zero_line_end_x = 0
+                            time_in_pixel_value = round((t_width / ax_w), 4)
+                            time_interval_for_data = round(1 / (ax_w / t_width), 4)
+                            start_x = ax_x
+                            start_y = 0
+                            value_to_draw = False
+                            display_count = int(check_timer / (t_width * 1000))
+                            total_display_count = int(max_video_length / (t_width * 1000))
+                            end_x = 0
+                            end_y = 0
+                            final_display_arr = display_arr[display_count][1]
+                            pointer_position_info_name = info_name + "_pointer_info"
+
+                            if len(final_display_arr) > 0:
+                                for ind, num in enumerate(final_display_arr):
+                                    if zero_display:
+                                        cv2.line(clone_frame, (zero_line_start_x, zero_line_y),
+                                                 (zero_line_end_x, zero_line_y),
+                                                 line_color, zero_line_thick)
+                                    if type(num[1]) == str:
+                                        pass
+
+                                    elif math.isnan(num[1]):
+                                        value_to_draw = False
+
+                                    else:
+                                        if not value_to_draw:
                                             scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
-                                            # scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
-                                            scaled_step = math.ceil((num[0] / 1000) / time_in_pixel_value)
+                                            scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
                                             start_x = ax_x + scaled_step - (display_count * ax_w)
                                             start_y = int(ax_y + ax_h - scaled_num)
                                             value_to_draw = True
-                                    else:
-                                        if start_x >= drawable_limit:
-                                            pen_draw = False
                                         else:
-                                            pen_draw = True
-                                        if pen_draw:
                                             scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
-                                            # scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
-                                            scaled_step = math.ceil((num[0] / 1000) / time_in_pixel_value)
-                                            end_x = ax_x + scaled_step - (display_count * ax_w)
+                                            scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
+                                            if display_count != total_display_count and ind == (
+                                                    len(final_display_arr) - 1):
+                                                end_x = ax_x + ax_w
+                                            else:
+                                                end_x = ax_x + scaled_step - (display_count * ax_w)
                                             end_y = int(ax_y + ax_h - scaled_num)
                                             cv2.line(clone_frame, (start_x, start_y), (end_x, end_y),
-                                                     line_color_array[i], line_thickness, cv2.LINE_AA)
-                                            if need_to_draw_pointer and round(num[0], 2) >= round(check_timer, 2):
-                                                draw_after = True
-                                                pointer_x = end_x
-                                                pointer_y = end_y
+                                                     line_color, line_thickness, cv2.LINE_AA)
+                                            if need_to_draw_pointer and (round(num[0] / 5, 0)) * 5 == round(check_timer,
+                                                                                                            0):
+                                                pointer_info_dict[pointer_position_info_name] = (end_x, end_y)
+
                                             start_x = end_x
                                             start_y = end_y
+                                pointer_position_tuple = pointer_info_dict[pointer_position_info_name]
+                                pointer_x = pointer_position_tuple[0]
+                                pointer_y = pointer_position_tuple[1]
+                                cv2.circle(clone_frame, (pointer_x, pointer_y), pointer_radius, pointer_color, -1)
 
-                            if draw_after:
-                                # draw_after = False
-                                cv2.circle(clone_frame,
-                                           (pointer_x, pointer_y), pointer_radius, pointer_color, -1)
-
-                    elif show_type == "static" and parent_index is not None:
-                        ax_pos = axes_info["position"]
-                        ax_x = ax_pos["x"]
-                        ax_y = ax_pos["y"]
-                        ax_w = ax_pos["width"]
-                        ax_h = ax_pos["height"]
-                        zero_line_start_x = ax_x
-                        zero_line_y_scale = int(number_to_scale(lower_limit, upper_limit, ax_h, 0))
-                        zero_line_y = int(ax_y + ax_h - zero_line_y_scale)
-                        zero_line_end_x = ax_x + ax_w
-                        time_in_pixel_value = round((t_width / ax_w), 4)
-                        # time_interval_for_data = round(1 / (ax_w / t_width), 4)
-                        start_x = ax_x
-                        start_y = 0
-                        value_to_draw = False
-                        display_count = int(check_timer / (t_width * 1000))
-                        total_display_count = int(max_video_length / (t_width * 1000))
-                        # end_x = 0
-                        # end_y = 0
-                        final_display_arr = display_arr[display_count][1]
-                        draw_after = False
-                        pointer_x = 0
-                        pointer_y = 0
-
-                        if len(final_display_arr) > 0:
-                            for ind, num in enumerate(final_display_arr):
-                                if zero_display:
-                                    cv2.line(clone_frame, (zero_line_start_x, zero_line_y),
-                                             (zero_line_end_x, zero_line_y),
-                                             line_color_array[i], zero_line_thick)
-
-                                if type(num[1]) == str:
-                                    pass
-
-                                elif math.isnan(num[1]):
-                                    value_to_draw = False
-
-                                else:
-                                    if not value_to_draw:
-                                        scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
-                                        scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
-                                        start_x = ax_x + scaled_step - (display_count * ax_w)
-                                        start_y = int(ax_y + ax_h - scaled_num)
-                                        value_to_draw = True
-
-                                    else:
-                                        scaled_num = number_to_scale(lower_limit, upper_limit, ax_h, num[1])
-                                        scaled_step = int(round(((num[0] / 1000) / time_in_pixel_value), 0))
-                                        if display_count != total_display_count and ind == (len(final_display_arr) - 1):
-                                            end_x = ax_x + ax_w
-                                        else:
-                                            end_x = ax_x + scaled_step - (display_count * ax_w)
-                                        end_y = int(ax_y + ax_h - scaled_num)
-                                        cv2.line(clone_frame, (start_x, start_y), (end_x, end_y),
-                                                 line_color_array[i], line_thickness, cv2.LINE_AA)
-                                        if need_to_draw_pointer and round(num[0], 2) == round(check_timer, 2):
-                                            draw_after = True
-                                            pointer_x = end_x
-                                            pointer_y = end_y
-                                        start_x = end_x
-                                        start_y = end_y
-
-                            if draw_after:
-                                # draw_after = False
-                                cv2.circle(clone_frame,
-                                           (pointer_x, pointer_y), pointer_radius, pointer_color, -1)
+                    else:
+                        print("\r")
+                        print("Drawable graph data is not found.")
+                        drawing_graph = False
 
                 for info in series_array:
                     search_string = info["parent_axes"]
